local _G = _G
local date = _G.date
local MAJOR_VERSION = 'GraveLevelBar'
local MINOR_VERSION = type('20110308041036') == 'number' and '20110308041036' or date('%Y%m%d%H%M%S')
local MODULE_NAME = 'GuildRep'

local LibStub = _G.LibStub
local AceAddon = LibStub('AceAddon-3.0')
local addon = AceAddon:GetAddon(MAJOR_VERSION)

local module = addon:createRepModule(MODULE_NAME, 'AceEvent-3.0')

local min = min

module.defaults.targetXP = module:calcXpForLevel(module:getTargetLevelMax())

module.oldEnable = module.OnEnable
function module:OnEnable()
    self:oldEnable()
    self:RegisterEvent('GUILD_REP_UPDATED')
    -- TODO find proper events
end

function module:Redraw()
    --addon:Debug("Guild XP Redraw")

    local xpOptions = self:getOptions()
    local barCache = self.cache

    local guildLevel = GetGuildLevel('player')
    local currentGlobalXP = self:getCurrentXP()
    -- update max settings if necessary
    if xpOptions.targetXP and currentGlobalXP > xpOptions.targetXP then
        xpOptions.targetXP = currentGlobalXP
    end
    local startGlobalXP = xpOptions.startXP
    local targetGlobalXP = xpOptions.targetXP
    local remainingGlobalXP = self:getRemainingWeeklyRep()
    local currentGlobalXPRatio =   (currentGlobalXP  -startGlobalXP)/(targetGlobalXP-startGlobalXP)
    local remainingGlobalXPRatio = (remainingGlobalXP              )/(targetGlobalXP-startGlobalXP)
    -- create textures
    assert(barCache and barCache.frame)
    local frame = barCache.frame
    local right = frame:GetRight()
    local left = frame:GetLeft()
    if not (right and left) then
        self:Invalidate()
        return
    end
    local width = frame:GetRight() - frame:GetLeft()
    -- texture for curent xp
    barCache.currentXPTexture = barCache.currentXPTexture or frame:CreateTexture(nil,'ARTWORK')
    local currentXPTexture = barCache.currentXPTexture
    currentXPTexture:ClearAllPoints()
    currentXPTexture:SetPoint('TOPLEFT')
    currentXPTexture:SetPoint('BOTTOM')
    currentXPTexture:SetWidth(currentGlobalXPRatio * width)
    currentXPTexture:SetTexture(0,0,1,1)

    -- texture for remaining xp
    barCache.remainingXPTexture = barCache.remainingXPTexture or frame:CreateTexture(nil,'ARTWORK')
    local remainingXPTexture = barCache.remainingXPTexture
    remainingXPTexture:ClearAllPoints()
    remainingXPTexture:SetPoint('TOP')
    remainingXPTexture:SetPoint('BOTTOM')
    remainingXPTexture:SetPoint('LEFT', currentXPTexture, 'RIGHT')
    local restedWidth = min(remainingGlobalXPRatio,1-currentGlobalXPRatio) * width
    remainingXPTexture:SetWidth(restedWidth)
    remainingXPTexture:SetTexture(0,0.5,0,1)

    -- textures and numbers for levels
    barCache.levelTextures = barCache.levelTextures or {}
    barCache.levelStrings = barCache.levelStrings or {}
    local levelTextures = barCache.levelTextures
    local levelStrings = barCache.levelStrings
    for level = 1,#REP_DATA do
        local xpForLeveL = self:calcXpForLevel(level)
        if startGlobalXP <= xpForLeveL and xpForLeveL <= targetGlobalXP then
            local levelGlobalXPRatio = (xpForLeveL-startGlobalXP)/(targetGlobalXP-startGlobalXP)
            -- textures
            levelTextures[level] = levelTextures[level] or frame:CreateTexture(frame:GetName()..'Level'..level,'OVERLAY')
            local levelTexture = levelTextures[level]
            levelTexture:ClearAllPoints()
            levelTexture:SetPoint('TOP')
            levelTexture:SetPoint('BOTTOM')
            if levelGlobalXPRatio < 1.0 then
                levelTexture:SetPoint('LEFT',frame,'LEFT',levelGlobalXPRatio * width,0)
            else
                levelTexture:SetPoint('RIGHT')
            end
            levelTexture:SetWidth(1)
            levelTexture:SetTexture(1,1,1,1)
            levelTexture:Show()
            -- numbers
            levelStrings[level] = levelStrings[level] or frame:CreateFontString(nil,OVERLAY)
            local levelString = levelStrings[level]
            local isValid = levelString:SetFont('Fonts\\FRIZQT__.TTF',xpOptions.size)
            assert(isValid)
            levelString:ClearAllPoints()
            if levelGlobalXPRatio > 0 then
                levelString:SetPoint('RIGHT',levelTexture,'LEFT')
                levelString:SetJustifyH('RIGHT')
            else
                levelString:SetPoint('LEFT',levelTexture,'RIGHT')
                levelString:SetJustifyH('LEFT')
            end
            levelString:SetJustifyV('MIDDLE')
            levelString:SetText(REP_NAMES[level])
            levelString:Show()
        else
            if levelTextures[level] then
                levelTextures[level]:Hide()
            end
            if levelStrings[level] then
                levelStrings[level]:Hide()
            end
        end
    end

    --update options table
    self:updateOptionsTable()
end

function module:getGuildRepData()
    local _, _, level, _, maxLocal, currentLocal, _,_,_,_,_,_,_, remaining, weekMax = GetGuildFactionInfo()
    assert(0 <= level and level <= #REP_DATA,"level="..level)
    return level, currentLocal, maxLocal, remaining, weekMax
end

function module:getCurrentXP()
    local level, currentLocal, _, _, _ =  self:getGuildRepData()
    return self:calcXpForLevel(level) + currentLocal
end

function module:getRemainingWeeklyRep()
    local _, _, _, remaining, _ = self:getGuildRepData()
    return remaining
end

function module:fillTooltip(tooltip)
    local currentGlobalXP = self:getCurrentXP()
    local targetGlobalXP = self:getOptions().targetXP or self:getTargetXPMax()
    local startGlobalXP = self:getOptions().startXP
    local deltaXP = targetGlobalXP-startGlobalXP
    local percent = (currentGlobalXP-startGlobalXP)/deltaXP
    local strPercent = format('%.2f%%', percent*100)
    tooltip:AddLine(currentGlobalXP ..'/' .. deltaXP .. '[' .. strPercent .. '%]')
end

